#!/bin/sh 
# This script redirects NMEA data from Share GPS to
# gpsd when using USB connections.  It implements adb
# forwarding for tcp data while gpsd listens.
#
# Requires: 
# adb from android sdk 
# gpsd - linux gps daemon 
# netstat - to verify adb forwarding is up
# 
# Run that script manually or better via udev rule 
#################### Some definitions to edit ################# 
# adb binary location 
LOCAL_PORT=20175 
MOBILE_PORT=50000
ADB=adb
LOG="YES"
LOGFILE=/tmp/adb_gps.log
USER=user
# Other stuff
NETSTAT=netstat
ADBGPSpid="/tmp/adb_gps_usb.pid"
# Select local port to bind with adb 
# gpsd binary location 
GPSD=gpsd
GPSDPID=/tmp/gpsd.pid 
# That's command to stop not needed gpsd service 
# If necessarily replace with your system command 
GPSDSTOP="/etc/init.d/gpsd stop" 
################## Definitins end ############################# 
[ "$LOG" = "YES" ] && LOG="$LOGFILE" && echo Loging enabled >> $LOG || LOG=/dev/null

stop_active_adb(){ 
# to be sure any other adb proccess doesn't mess our connection 
[ -n "`ps aux | grep adb`" ] && $ADB kill-server 2>>$LOG
}
 
# Enable adb tcp port forwarding 
connect_adb(){ 
runuser -l $USER -c "$ADB start-server"
sleep 5
#$ADB devices >> $LOG
$ADB forward tcp:$LOCAL_PORT tcp:$MOBILE_PORT 2>>$LOG
} 

###########
# Stop
###########
stop(){
kill -3 `cat $GPSDPID` 2>>$LOG
stop_active_adb
# Close possile stale script may be even self
for i in `cat $ADBGPSpid`;do
	kill -3 $i 2>>$LOG
# Next force cleanup
	kill -9 $i 2>>$LOG
	done
rm -f $ADBGPSpid
exit 0 
} 

redirect_data(){ 
# Command to check if there is connection to ShareGPS service 
ADBSTATUS=`$NETSTAT -nlp | grep $LOCAL_PORT | grep adb`
echo "Start ADBSTATUS=$ADBSTATUS" >>$LOG
# Run until there is active connection on selected port 
while [ -n "$ADBSTATUS" ]; do 
# When above redirection fail but adb is still active 
# try to redirect data again 
sleep 2
ADBSTATUS=`$NETSTAT -nlp | grep $LOCAL_PORT | grep adb`
#echo "ADBSTATUS=$ADBSTATUS `date`" >>$LOG
done
# When adb connection is lost stop and clean 
stop 
} 

########
# Start
########
start(){
# Write script pid
echo $$ >> $ADBGPSpid
stop_active_adb
connect_adb
# Be sure, system gpsd service is not runnig 
$GPSDSTOP 
# Start gpsd 
$GPSD -b -P $GPSDPID tcp://localhost:$LOCAL_PORT 
redirect_data 
} 

#######
# Main
#######
case $1 in
	start)
        echo "Start at `date`
############################" >>$LOG
	start
	;;
	stop) 
	echo "Stop at `date`
###########################" >>$LOG
	stop
	;;
	restart)
	stop
	start
	;;
	*)
	echo "Usage start|stop|restart"
esac

